import java.io.*;

public class MapSwapper
{
    private final int m_maxSupportedOS = OperatingSystem.AsInt(OperatingSystem.MAX_SUPPORTED_OS);
    private final int m_currentOS = OperatingSystem.AsInt(OperatingSystem.GetCurrentOS());
    private final String[] m_blizzardCacheLocation;
    
    private final String[] m_cacheFile;
    
    private String mapFile;
    boolean m_swapped = false;
    
    private MapSwapper()
    {
        // Setup OS-dependent Map paths.
        m_blizzardCacheLocation = new String[m_maxSupportedOS];
        m_blizzardCacheLocation[OperatingSystem.AsInt(OperatingSystem.WINDOWS)] = OperatingSystem.GetApplicationDataDirectory() +
                                                                                  "\\Blizzard Entertainment\\Battle.net\\Cache\\";                                                                           
        m_blizzardCacheLocation[OperatingSystem.AsInt(OperatingSystem.MACOSX)] =  OperatingSystem.GetApplicationDataDirectory() +
                                                                                  "/Application Support/Battle.net/Cache/";
        m_cacheFile = new String[m_maxSupportedOS];  
        // yes, these hashes are all the same atm. but there was a rare instance where they weren't. wtf? yeah. yeah....
        m_cacheFile[OperatingSystem.AsInt(OperatingSystem.WINDOWS)] = "44/75/4475ddf83ebd5a0072d29a4009374cf2187f963acede493edfa5f6a5b07c4f49.s2ma";
        m_cacheFile[OperatingSystem.AsInt(OperatingSystem.MACOSX)] = "44/75/4475ddf83ebd5a0072d29a4009374cf2187f963acede493edfa5f6a5b07c4f49.s2ma";
    }
    
    public void setMapFile(String s)
    {
        mapFile = s;
    }
    
    private void moveFile(String from, String to)
    {
        File f = new File(from);
        f.renameTo(new File(to));
    }
    
    public String getCacheFileName()
    {
        String name = m_cacheFile[m_currentOS]; 
        
        // Remove 44/75/ prefix or whatnot.
        int p = name.lastIndexOf('/');
        name = name.substring(p+1);
        return name;
    }
    
    public boolean swapped()
    {
        return m_swapped;
    }
    
    public void swap()
    {
        // abort if the move operation will fail.
        if( !hasLostTemple() || !hasSaveDirectory() )
            return;
            
        moveFile( m_blizzardCacheLocation[m_currentOS] + m_cacheFile[m_currentOS], OperatingSystem.GetDataDirectoryForOS() + getCacheFileName() );
        moveFile( mapFile, m_blizzardCacheLocation[m_currentOS] + m_cacheFile[m_currentOS] );
        m_swapped = true;
    }
    
    public void restore()
    {
        // abort if the move operation will fail.
        if( !hasLostTemple() || !hasSaveDirectory() )
            return;
            
        moveFile( m_blizzardCacheLocation[m_currentOS] + m_cacheFile[m_currentOS], mapFile );
        moveFile( OperatingSystem.GetDataDirectoryForOS() + getCacheFileName(), m_blizzardCacheLocation[m_currentOS] + m_cacheFile[m_currentOS] );
        
        m_swapped = false;
    }
    
    public boolean hasLostTemple()
    {
        // Find and check for existance of cacheFile.
        File f = new File( m_blizzardCacheLocation[m_currentOS] + m_cacheFile[m_currentOS] );
        return f.isFile();
    }
    public boolean hasSaveDirectory()
    {
        File f = new File( OperatingSystem.GetDataDirectoryForOS() );
        if( !f.isDirectory() ) // attempt to make our directory then.
        {
            return f.mkdir();
        }
        
        return true;
    }
    
    private static MapSwapper m_swapper;
    
    public static MapSwapper getSingleton()
    {
        if( m_swapper == null)
            m_swapper = new MapSwapper();
            
        return m_swapper;
    }
}